@@ -1,36 +1,63 @@
-# -*- coding: utf-8 -*-
-"""Shared SQL utilities"""
+# -*- encoding: utf-8 -*-
+"""Logical to Physical Mappers"""
 
-import collections
-from cubes.common import get_logger
-from cubes.errors import *
+from __future__ import absolute_import
+
+from collections import namedtuple
+
+from ...logging import get_logger
+from ...errors import *
+from ...mapper import Mapper
+from ...model import AttributeBase, Attribute
+from ... import compat
 
 __all__ = (
-    "Mapper",
     "SnowflakeMapper",
     "DenormalizedMapper",
-    "PhysicalReference",
-    "Join",
+    "TableColumnReference",
+    "TableJoin",
     "coalesce_physical",
+    "PhysicalAttribute",
     "DEFAULT_KEY_FIELD"
 )
 
 DEFAULT_KEY_FIELD = "id"
 
-# FIXME: list of required fixes to the AttributeMapper:
-#
-# * remove need to use dimension where attribute object is passed (dimension
-#   should now be part of the attribute)
-
 """Physical reference to a table column. Note that the table might be an
 aliased table name as specified in relevant join."""
-PhysicalReference = collections.namedtuple("PhysicalReference",
-                                    ["schema", "table", "column", "extract"])
+TableColumnReference = namedtuple("TableColumnReference",
+                                    ["schema", "table", "column", "extract", "func", "expr", "condition"])
 
-"""Table join specification. `master` and `detail` are PhysicalReference
-(3-item) tuples"""
-Join = collections.namedtuple("Join",
-                                    ["master", "detail", "alias"])
+"""Table join specification. `master` and `detail` are TableColumnReference
+tuples. `method` denotes which table members should be considered in the join:
+*master* – all master members (left outer join), *detail* – all detail members
+(right outer join) and *match* – members must match (inner join)."""
+TableJoin = namedtuple("TableJoin",
+                                    ["master", "detail", "alias", "method"])
+
+
+SnowflakeTable = namedtuple("SnowflakeTable",
+                            ["schema", "table", "outlets"])
+
+_join_method_order = {"detail":0, "master":1, "match": 2}
+
+# Note to developers: Used for internal purposes to represent a physical table
+# column. Currently used only in the PTD condition.
+class PhysicalAttribute(AttributeBase):
+    def __init__(self, name, label=None, description=None, order=None,
+                 info=None, format=None, table=None, missing_value=None,
+                 **kwargs):
+        super(PhysicalAttribute, self).__init__(name=name, label=label,
+                                        description=description, order=order,
+                                        info=info, format=format,
+                                        missing_value=missing_value)
+        self.table = table
+
+    def ref(self, simplify=True, locale=None):
+        if self.table is not None:
+            return "%s.%s" % (self.table, self.name)
+        else:
+            return self.name
 
 def coalesce_physical(ref, default_table=None, schema=None):
     """Coalesce physical reference `ref` which might be:
@@ -38,11 +65,11 @@ def coalesce_physical(ref, default_table=None, schema=None):
     * a string in form ``"table.column"``
     * a list in form ``(table, column)``
     * a list in form ``(schema, table, column)``
-    * a dictionary with keys: ``schema``, ``table``, ``column`` where
+    * a dictionary with keys: ``schema``, ``table``, ``column``, ``extract``, ``func``, ``expr``, ``condition`` where
       ``column`` is required, the rest are optional
 
-    Returns tuple (`schema`, `table`, `column`, `extract`), which is a named
-    tuple `PhysicalReference`.
+    Returns tuple (`schema`, `table`, `column`, `extract`, `func`, `expr`, `condition`), which is a named
+    tuple `TableColumnReference`.
 
     If no table is specified in reference and `default_table` is not
     ``None``, then `default_table` will be used.
@@ -53,189 +80,33 @@ def coalesce_physical(ref, default_table=None, schema=None):
 
     """
 
-    if isinstance(ref, basestring):
+    if isinstance(ref, compat.string_type):
         split = ref.split(".")
 
         if len(split) > 1:
             dim_name = split[0]
             attr_name = ".".join(split[1:])
-            return PhysicalReference(schema, dim_name, attr_name, None)
+            return TableColumnReference(schema, dim_name, attr_name, None, None, None, None)
         else:
-            return PhysicalReference(schema, default_table, ref, None)
+            return TableColumnReference(schema, default_table, ref, None, None, None, None)
     elif isinstance(ref, dict):
-        return PhysicalReference(ref.get("schema", schema),
+        return TableColumnReference(ref.get("schema", schema),
                                  ref.get("table", default_table),
                                  ref.get("column"),
-                                 ref.get("extract"))
+                                 ref.get("extract"),
+                                 ref.get("func"),
+                                 ref.get("expr"),
+                                 ref.get("condition"))
     else:
         if len(ref) == 2:
-            return PhysicalReference(schema, ref[0], ref[1], None)
+            return TableColumnReference(schema, ref[0], ref[1], None, None, None, None)
         elif len(ref) == 3:
-            return PhysicalReference(ref[0], ref[1], ref[2], None)
+            return TableColumnReference(ref[0], ref[1], ref[2], None, None, None, None)
         else:
             raise BackendError("Number of items in table reference should "\
                                "be 2 (table, column) or 3 (schema, table, column)")
 
 
-class Mapper(object):
-    """Mapper is core clas for translating logical model to physical
-    database schema.
-    """
-    # WARNING: do not put any SQL/engine/connection related stuff into this
-    # class yet. It might be moved to the cubes as one of top-level modules
-    # and subclassed here.
-
-    def __init__(self, cube, locale=None, schema=None, fact_name=None,
-                 **options):
-        """Abstract class for mappers which maps logical references to
-        physical references (tables and columns).
-
-        Attributes:
-
-        * `cube` - mapped cube
-        * `simplify_dimension_references` – references for flat dimensions 
-          (with one level and no details) will be just dimension names, no 
-          attribute name. Might be useful when using single-table schema, for 
-          example, with couple of one-column dimensions.
-        * `fact_name` – fact name, if not specified then `cube.name` is used
-        * `schema` – default database schema
-
-        """
-
-        super(Mapper, self).__init__()
-
-        if cube == None:
-            raise Exception("Cube for mapper should not be None.")
-
-        self.logger = get_logger()
-
-        self.cube = cube
-        self.locale = locale
-
-        fact_prefix = options.get("fact_prefix") or ""
-        self.fact_name = fact_name or self.cube.fact or fact_prefix+self.cube.name
-        self.schema=schema
-
-        if "simplify_dimension_references" in options:
-            self.simplify_dimension_references = options["simplify_dimension_references"]
-        else:
-            self.simplify_dimension_references = True
-
-        self._collect_attributes()
-
-    def _collect_attributes(self):
-        """Collect all cube attributes and create a dictionary where keys are
-        logical references and values are `cubes.model.Attribute` objects.
-        This method should be used after each cube or mappings change.
-        """
-
-        self.attributes = collections.OrderedDict()
-
-        for attr in self.cube.measures:
-            self.attributes[self.logical(attr)] = attr
-
-        for attr in self.cube.details:
-            self.attributes[self.logical(attr)] = attr
-
-        for dim in self.cube.dimensions:
-            for attr in dim.all_attributes():
-                if not attr.dimension:
-                    raise Exception("No dimension in attr %s" % attr)
-                self.attributes[self.logical(attr)] = attr
-
-    def all_attributes(self, expand_locales=False):
-        """Return a list of all attributes of a cube. If `expand_locales` is
-        ``True``, then localized logical reference is returned for each
-        attribute's locale."""
-        return self.attributes.values()
-
-    def attribute(self, name):
-        """Returns an attribute with logical reference `name`. """
-        # TODO: If attribute is not found, returns `None` (yes or no?)
-
-        return self.attributes[name]
-
-    def logical(self, attribute, locale=None):
-        """Returns logical reference as string for `attribute` in `dimension`.
-        If `dimension` is ``Null`` then fact table is assumed. The logical
-        reference might have following forms:
-
-        * ``dimension.attribute`` - dimension attribute
-        * ``attribute`` - fact measure or detail
-
-        If `simplify_dimension_references` is ``True`` then references for
-        flat dimensios without details is `dimension`.
-        
-        If `locale` is specified, then locale is added to the reference. This
-        is used by backends and other mappers, it has no real use in end-user
-        browsing.
-        """
-
-        dimension = attribute.dimension
-
-        if dimension:
-            simplify = self.simplify_dimension_references and \
-                               (dimension.is_flat and not dimension.has_details)
-        else:
-            simplify = False
-            
-        reference = attribute.ref(simplify, locale)
-        
-        return reference
-
-    def split_logical(self, reference):
-        """Returns tuple (`dimension`, `attribute`) from `logical_reference` string. Syntax
-        of the string is: ``dimensions.attribute``."""
-        
-        split = reference.split(".")
-
-        if len(split) > 1:
-            dim_name = split[0]
-            attr_name = ".".join(split[1:])
-            return (dim_name, attr_name)
-        else:
-            return (None, reference)
-
-    def physical(self, attribute, locale=None):
-        """Returns physical reference as tuple for `attribute`, which should
-        be an instance of :class:`cubes.model.Attribute`. If there is no
-        dimension specified in attribute, then fact table is assumed. The
-        returned tuple has structure: (`schema`, `table`, `column`).
-
-        This method should be implemented by `Mapper` subclasses.
-        """
-        
-        raise NotImplementedError
-                
-    def map_attributes(self, attributes, expand_locales=False):
-        """Convert `attributes` to physical attributes. If `expand_locales` is
-        ``True`` then physical reference for every attribute locale is
-        returned."""
-        
-        if expand_locales:
-            physical_attrs = []
-
-            for attr in attributes:
-                if attr.locales:
-                    refs = [self.physical(attr, locale) for locale in attr.locales]
-                else:
-                    refs = [self.physical(attr)]
-                physical_attrs += refs
-        else:
-            physical_attrs = [self.physical(attr) for attr in attributes]
-            
-        return physical_attrs
-
-    def relevant_joins(self, attributes):
-        """Get relevant joins to the attributes - list of joins that 
-        are required to be able to acces specified attributes. `attributes`
-        is a list of three element tuples: (`schema`, `table`, `attribute`).
-
-        Subclasses sohuld implement this method.
-        """
-        
-        raise NotImplementedError
-
 class SnowflakeMapper(Mapper):
     """Mapper is core clas for translating logical model to physical
     database schema.
@@ -245,8 +116,8 @@ class SnowflakeMapper(Mapper):
     # and subclassed here.
 
     def __init__(self, cube, mappings=None, locale=None, schema=None,
-                    fact_name=None, dimension_prefix=None, joins=None,
-                    **options):
+                 fact_name=None, dimension_prefix=None, dimension_suffix=None,
+                 joins=None, dimension_schema=None, **options):
 
         """A snowflake schema mapper for a cube. The mapper creates required
         joins, resolves table names and maps logical references to tables and
@@ -256,15 +127,18 @@ class SnowflakeMapper(Mapper):
 
         * `cube` - mapped cube
         * `mappings` – dictionary containing mappings
-        * `simplify_dimension_references` – references for flat dimensions 
-          (with one level and no details) will be just dimension names, no 
-          attribute name. Might be useful when using single-table schema, for 
+        * `simplify_dimension_references` – references for flat dimensions
+          (with one level and no details) will be just dimension names, no
+          attribute name. Might be useful when using single-table schema, for
           example, with couple of one-column dimensions.
-        * `dimension_prefix` – default prefix of dimension tables, if 
+        * `dimension_prefix` – default prefix of dimension tables, if
+          default table name is used in physical reference construction
+        * `dimension_suffix` – default suffix of dimension tables, if
           default table name is used in physical reference construction
         * `fact_name` – fact name, if not specified then `cube.name` is used
         * `schema` – default database schema
-        * `dimension_prefix` – prefix for dimension tables
+        * `dimension_schema` – schema whre dimension tables are stored (if
+          different than fact table schema)
 
         `mappings` is a dictionary where keys are logical attribute references
         and values are table column references. The keys are mostly in the
@@ -283,12 +157,18 @@ class SnowflakeMapper(Mapper):
 
         """
 
-        super(SnowflakeMapper, self).__init__(cube, locale=locale,
-                                        schema=schema, fact_name=fact_name,
-                                        **options)
+        super(SnowflakeMapper, self).__init__(cube, locale=locale, **options)
 
         self.mappings = mappings or cube.mappings
-        self.dimension_prefix = dimension_prefix
+        self.dimension_prefix = dimension_prefix or ""
+        self.dimension_suffix = dimension_suffix or ""
+        self.dimension_schema = dimension_schema
+
+        fact_prefix = options.get("fact_prefix") or ""
+        fact_suffix = options.get("fact_suffix") or ""
+        self.fact_name = fact_name or self.cube.fact or "%s%s%s" % \
+                            (fact_prefix, self.cube.basename, fact_suffix)
+        self.schema = schema
 
         self._collect_joins(joins or cube.joins)
 
@@ -305,8 +185,12 @@ class SnowflakeMapper(Mapper):
         for join in joins:
             master = coalesce_physical(join["master"],self.fact_name,schema=self.schema)
             detail = coalesce_physical(join["detail"],schema=self.schema)
-            self.logger.debug("collecting join %s - %s" % (tuple(master), tuple(detail)))
-            self.joins.append(Join(master, detail, join.get("alias")))
+
+            self.logger.debug("collecting join %s -> %s" % (tuple(master), tuple(detail)))
+            method = join.get("method", "match").lower()
+
+            self.joins.append(TableJoin(master, detail, join.get("alias"),
+                                        method))
 
     def physical(self, attribute, locale=None):
         """Returns physical reference as tuple for `attribute`, which should
@@ -351,10 +235,19 @@ class SnowflakeMapper(Mapper):
                     table name is fact table name
         """
 
-        # FIXME: we need schema as well, see Issue #43
+        if isinstance(attribute, Attribute) and attribute.dimension:
+            schema = self.dimension_schema or self.schema
+        else:
+            schema = self.schema
+
+        if isinstance(attribute, PhysicalAttribute):
+            reference = TableColumnReference(schema,
+                                             attribute.table,
+                                             attribute.name,
+                                             None, None, None, None)
+            return reference
 
         reference = None
-        dimension = attribute.dimension
 
         # Fix locale: if attribute is not localized, use none, if it is
         # localized, then use specified if exists otherwise use default
@@ -362,7 +255,7 @@ class SnowflakeMapper(Mapper):
 
         locale = locale or self.locale
 
-        if attribute.locales:
+        if attribute.is_localizable():
             locale = locale if locale in attribute.locales \
                                 else attribute.locales[0]
         else:
@@ -372,7 +265,7 @@ class SnowflakeMapper(Mapper):
         if self.cube.mappings:
             logical = self.logical(attribute, locale)
 
-            # TODO: should default to non-localized reference if no mapping 
+            # TODO: should default to non-localized reference if no mapping
             # was found?
             mapped_ref = self.cube.mappings.get(logical)
 
@@ -387,17 +280,16 @@ class SnowflakeMapper(Mapper):
             if locale:
                 column_name += "_" + locale
 
+            # TODO: temporarily preserved. it should be attribute.owner
+            dimension = attribute.dimension
             if dimension and not (self.simplify_dimension_references \
-                                   and (dimension.is_flat 
+                                   and (dimension.is_flat
                                         and not dimension.has_details)):
-                table_name = str(dimension)
-                if self.dimension_prefix:
-                    table_name = self.dimension_prefix + table_name
-
+                table_name = "%s%s%s" % (self.dimension_prefix, dimension, self.dimension_suffix)
             else:
                 table_name = self.fact_name
 
-            reference = PhysicalReference(self.schema, table_name, column_name, None)
+            reference = TableColumnReference(schema, table_name, column_name, None, None, None, None)
 
         return reference
 
@@ -419,8 +311,8 @@ class SnowflakeMapper(Mapper):
         }
 
         for join in self.joins:
-            if not join.detail.table or join.detail.table == self.fact_name:
-                raise BackendError("Detail table name should be present and should not be a fact table.")
+            if not join.detail.table or (join.detail.table == self.fact_name and not join.alias):
+                raise BackendError("Detail table name should be present and should not be a fact table unless aliased.")
 
             ref = (join.master.schema, join.master.table)
             tables[ref] = ref
@@ -430,8 +322,35 @@ class SnowflakeMapper(Mapper):
 
         return tables
 
-    def relevant_joins(self, attributes):
-        """Get relevant joins to the attributes - list of joins that 
+    def physical_references(self, attributes, expand_locales=False):
+        """Convert `attributes` to physical attributes. If `expand_locales` is
+        ``True`` then physical reference for every attribute locale is
+        returned."""
+
+        if expand_locales:
+            physical_attrs = []
+
+            for attr in attributes:
+                if attr.is_localizable():
+                    refs = [self.physical(attr, locale) for locale in attr.locales]
+                else:
+                    refs = [self.physical(attr)]
+                physical_attrs += refs
+        else:
+            physical_attrs = [self.physical(attr) for attr in attributes]
+
+        return physical_attrs
+
+    def tables_for_attributes(self, attributes, expand_locales=False):
+        """Returns a list of tables – tuples (`schema`, `table`) that contain
+        `attributes`."""
+
+        references = self.physical_references(attributes, expand_locales)
+        tables = [(ref[0], ref[1]) for ref in references]
+        return tables
+
+    def relevant_joins(self, attributes, expand_locales=False):
+        """Get relevant joins to the attributes - list of joins that
         are required to be able to acces specified attributes. `attributes`
         is a list of three element tuples: (`schema`, `table`, `attribute`).
         """
@@ -439,44 +358,58 @@ class SnowflakeMapper(Mapper):
         # Attribute: (schema, table, column)
         # Join: ((schema, table, column), (schema, table, column), alias)
 
-        self.logger.debug("getting relevant joins for %s attributes" % len(attributes))
+        # self.logger.debug("getting relevant joins for %s attributes" % len(attributes))
 
         if not self.joins:
             self.logger.debug("no joins to be searched for")
 
-        tables_to_join = {(ref[0], ref[1]) for ref in attributes}
+        tables_to_join = set(self.tables_for_attributes(attributes,
+                                                        expand_locales))
         joined_tables = set()
-        joined_tables.add( (self.schema, self.fact_name) )
+        fact_table = (self.schema, self.fact_name)
+        joined_tables.add( fact_table )
 
         joins = []
-        self.logger.debug("tables to join: %s" % tables_to_join)
+        # self.logger.debug("tables to join: %s" % list(tables_to_join))
 
         while tables_to_join:
             table = tables_to_join.pop()
             # self.logger.debug("joining table %s" % (table, ))
 
-            for join in self.joins:
-                # self.logger.debug("testing join: %s" % (join, ))
-                # print "--- testing join: %s" % (join, )
+            joined = False
+            for order, join in enumerate(self.joins):
                 master = (join.master.schema, join.master.table)
                 detail = (join.detail.schema, join.alias or join.detail.table)
+                # self.logger.debug("testing join: %s->%s" % (master,detail))
 
                 if table == detail:
                     # self.logger.debug("detail matches")
-                    joins.append(join)
+                    # Preserve join order
+                    # TODO: temporary way of ordering according to match
+                    method_order = _join_method_order.get(join.method, 99)
+                    joins.append( (method_order, order, join) )
 
                     if master not in joined_tables:
-                        self.logger.debug("adding master %s to be joined" % (master, ))
+                        # self.logger.debug("adding master %s to be joined" % (master, ))
                         tables_to_join.add(master)
 
-                    self.logger.debug("joined detail %s" % (detail, ) )
+                    # self.logger.debug("joined detail %s" % (detail, ) )
                     joined_tables.add(detail)
+                    joined = True
                     break
 
-        # FIXME: is join order important? if yes, we should sort them here
-        self.logger.debug("%s tables joined (of %s joins)" % (len(joins), len(self.joins)) )
-        self.logger.debug("joined tables: %s" % joins)
+            if joins and not joined and table != fact_table:
+                self.logger.warning("No table joined for %s" % (table, ))
 
+        # self.logger.debug("%s tables joined (of %s joins)" % (len(joins), len(self.joins)) )
+
+        # Sort joins according to original order specified in the model
+        joins.sort()
+        self.logger.debug("joined tables: %s" % ([join[2].detail.table for join in
+                                                                joins], ) )
+
+        # Retrieve actual joins from tuples. Remember? We preserved order.
+        joins = [join[2] for join in joins]
         return joins
 
 
@@ -498,14 +431,15 @@ class DenormalizedMapper(Mapper):
         * `fact_schema` – database schema for the original fact table
         """
 
-        super(DenormalizedMapper, self).__init__(cube, locale=locale, 
+        super(DenormalizedMapper, self).__init__(cube, locale=locale,
                                         schema=schema, fact_name=fact_name)
 
         dview_prefix = denormalized_view_prefix or ""
-        
+
         # FIXME: this hides original fact name, we do not want that
-        
-        self.fact_name = options.get("denormalized_view") or dview_prefix + self.cube.name
+
+        self.fact_name = options.get("denormalized_view") or dview_prefix + \
+                            self.cube.basename
         self.fact_schema = self.schema
         self.schema = self.schema or denormalized_view_schema
 
@@ -522,12 +456,12 @@ class DenormalizedMapper(Mapper):
                 locale = None
         except:
             locale = None
-            
+
         column_name = self.logical(attribute, locale)
-        reference = PhysicalReference(self.schema,
-                                      self.fact_name,
-                                      column_name,
-                                      None)
+        reference = TableColumnReference(self.schema,
+                                          self.fact_name,
+                                          column_name,
+                                          None, None, None, None)
 
         return reference
 
@@ -539,3 +473,4 @@ class DenormalizedMapper(Mapper):
         self.logger.debug("getting relevant joins: not needed for denormalized table")
 
         return []
+
