@@ -4,6 +4,8 @@ from flask import render_template, redirect
 from jinja2 import Template
 import json, re
 from functools import wraps
+import sys
+import traceback
 
 from ..workspace import Workspace, SLICER_INFO_KEYS
 from ..browser import Cell, SPLIT_DIMENSION_NAME, cut_from_dict
@@ -73,6 +75,7 @@ def _store_option(config, option, default, type_=None, allowed=None,
 
     setattr(current_app.slicer, option, value)
 
+
 @slicer.record_once
 def initialize_slicer(state):
     """Create the workspace and configure the application context from the
@@ -87,7 +90,15 @@ def initialize_slicer(state):
         params = CustomDict()
         current_app.slicer = params
         current_app.slicer.config = config
-        current_app.cubes_workspace = Workspace(config)
+
+        # FIXME: this is a workaround, see Workspace note about _options
+        if "cubes_root" in state.options:
+            _options = {"cubes_root": state.options["cubes_root"]}
+        else:
+            _options = {}
+
+        if not hasattr(current_app, 'cubes_workspace'):
+            current_app.cubes_workspace = Workspace(config, **_options)
 
         # Configure the application
         _store_option(config, "prettyprint", False, "bool")
@@ -191,7 +202,23 @@ def page_not_found(e):
 
 @slicer.errorhandler(InternalError)
 def server_error(e):
-    logger.error("Internal error: %s" % e)
+
+    (exc_type, exc_value, exc_traceback) = sys.exc_info()
+    exc_name = exc_type.__name__
+    logger.error("Internal Cubes error ({}): {}".format(exc_name, exc_value))
+
+    tb = traceback.format_exception(exc_type, exc_value,
+                                    exc_traceback)
+    logger.debug("Exception stack trace:\n{}".format("".join(tb)))
+
+    error = {
+        "error": "internal_server_error",
+        "message": "Internal server error",
+        "hint": "Server administrators can learn more about the error from "
+                "the error logs (even more if they have 'debug' level)"
+    }
+
+    return jsonify(error), 500
 
 # Endpoints
 # =========
@@ -273,6 +300,7 @@ def cube_model(cube_name):
                               hierarchy_limits=hier_limits)
 
     response["features"] = workspace.cube_features(g.cube)
+
     return jsonify(response)
 
 
