@@ -73,7 +73,8 @@ class ModelObjectInfo(object):
         return self.instances[key]
 
 class Workspace(object):
-    def __init__(self, config=None, stores=None, load_base_model=True):
+    def __init__(self, config=None, stores=None, load_base_model=True,
+                 **_options):
         """Creates a workspace. `config` should be a `ConfigParser` or a
         path to a config file. `stores` should be a dictionary of store
         configurations, a `ConfigParser` or a path to a ``stores.ini`` file.
@@ -94,6 +95,9 @@ class Workspace(object):
           are language to translation path mappings.
         """
 
+        # FIXME: **_options is temporary solution/workaround before we get
+        # better configuration. Used internally. Don't use!
+
         if isinstance(config, compat.string_type):
             cp = ConfigParser()
             try:
@@ -127,6 +131,10 @@ class Workspace(object):
         # Set the default models path
         if config.has_option("workspace", "root_directory"):
             self.root_dir = config.get("workspace", "root_directory")
+        elif "cubes_root" in _options:
+            # FIXME: this is quick workaround, see note at the beginning of
+            # this method
+            self.root_dir = _options["cubes_root"]
         else:
             self.root_dir = ""
 
@@ -287,6 +295,7 @@ class Workspace(object):
         if config.has_option("workspace", "authorization"):
             auth_type = config.get("workspace", "authorization")
             options = dict(config.items("authorization"))
+            options["cubes_root"] = self.root_dir
             self.authorizer = extensions.authorizer(auth_type, **options)
         else:
             self.authorizer = None
@@ -296,7 +305,6 @@ class Workspace(object):
 
         # Models are searched in:
         # [model]
-        # [workspace] model <- depreciated!
         # [models] * <- depreciated!
         # TODO: add this for nicer zero-conf
         # root/model.json
@@ -311,10 +319,6 @@ class Workspace(object):
             path = config.get("model", "path")
             models.append(("main", path))
 
-        # TODO: Depreciated before even being used
-        if config.has_option("workspace", "model"):
-            models.append( ("main", config.get("workspace", "model")) )
-
         # TODO: Depreciate this too
         if config.has_section("models"):
             models += config.items("models")
@@ -416,8 +420,9 @@ class Workspace(object):
 
         return store_name
 
-    # TODO: this is very complicated process, needs simplification
-    # TODO: change this to: import(name, info, provider, store, languages, ns)
+    # TODO: this is very confusing process, needs simplification
+    # TODO: change this to: add_model_provider(provider, info, store, languages, ns)
+
     def import_model(self, model=None, provider=None, store=None,
                      translations=None, namespace=None):
         """Registers the `model` in the workspace. `model` can be a
@@ -439,10 +444,6 @@ class Workspace(object):
         :meth:`cubes.Workspace.cube` or :meth:`cubes.Workspace.dimension` is
         called.
         """
-        if store and not isinstance(store, compat.string_type):
-            raise ArgumentError("Store should be provided by name "
-                                "(as a string).")
-
         # 1. Metadata
         # -----------
         # Make sure that the metadata is a dictionary
@@ -586,23 +587,7 @@ class Workspace(object):
             trans = context.object_localization("cubes", cube.name)
             cube = cube.localized(trans)
 
-        nsname = nsname or "default"
-
-        # Get the translation
-        # 1. look if there is translation for the namespace
-        # 2. find cube in the translation
-        # 3. translate if we have something
-        # translation = None
-        # ns_translation = self.translation(locale, nsname)
-        # if ns_translation:
-        #    cubes_translation = ns_translation.get("cubes")
-        #    if cubes_translation:
-        #        translation = cubes_translation.get(basename)
-
-        # if translation:
-        #    cube = copy(cube)
-        #    cube.localize(translation)
-
+        # Cache the cube
         self._cubes[cube_key] = cube
 
         return cube
